#!/usr/bin/python

import argparse
import logging
import os
import shutil
import subprocess
import sys


def findAllFile(dir, ext):
    files = []
    try:
        list = os.listdir(dir)
        for f in list:
            fullname = os.path.join(dir, f)
            if not os.path.isfile(fullname) or f.startswith("."):
                continue
            if ext == "":
                files.append(fullname)
            elif f.endswith(ext):
                files.append(fullname)
    except:
        pass
    return sorted(files)


def main(argv):
    inputdir = ""
    outputdir = ""
    language = "chi"
    title = ""
    fonts = ""
    nrename = False
    parser = argparse.ArgumentParser()
    parser.add_argument("-i", "--input", help="input directory", required=True)
    parser.add_argument("-o", "--output", help="output directory")
    parser.add_argument("-l", "--language", help="language")
    parser.add_argument("-t", "--title", help="subtitle title")
    parser.add_argument("-f", "--fonts", help="fonts dir")
    parser.add_argument("-n", "--nrename", help="not rename font", action="store_true")
    parser.add_argument("-nck", "--not-check", help="disable check mode", action="store_true")
    parser.add_argument("-ncks", "--not-strict", help="disable strict mode for check.", action="store_true")

    args = parser.parse_args()
    if args.input:
        inputdir = args.input
    if args.output:
        outputdir = args.output
    if args.language:
        language = args.language
    if args.title:
        title = args.title
    if args.fonts:
        fonts = args.fonts
    nrename = args.nrename
    not_check = args.not_check
    not_strict = args.not_strict
    logging.basicConfig(level=logging.INFO, datefmt='%Y/%m/%d %H:%M:%S', format='%(asctime)s %(message)s')
    asses = findAllFile(inputdir, ".ass")
    _l = len(asses)
    if _l == 0:
        logging.info('No ass files found in "%s".' % inputdir)
        return
    i = 0
    for ass in asses:
        n = os.path.split(ass)[1][:-4]
        p = os.path.join(outputdir, n)
        _args = ["mkvtool", "s", ass, "-o", p]
        if nrename:
            _args.append("--no-font-rename")
        if fonts != "":
            _args.extend(["-f", fonts])
        if not_check:
            _args.append("--no-font-check")
        if not_strict:
            _args.append("--no-font-check-strict")
        subprocess.run(_args)
        _p = os.path.join(p, "subsetted")
        if os.path.exists(_p):
            _asses = findAllFile(_p, ".ass")
            if len(_asses) > 0:
                _ass = _asses[0]
                _fonts = [i for i in findAllFile(_p, "") if not i.endswith(".ass")]
                _mks = os.path.join(outputdir, "%s.mks" % n)
                _args = ["mkvmerge",
                         "--output", _mks,
                         "--language", ("0:%s" % language)]
                if title != "":
                    _args.extend(["--track-name", "0:%s" % title])
                for font in _fonts:
                    _args.extend(["--attach-file", font])
                _args.append(_ass)
                subprocess.run(_args, stdout=subprocess.DEVNULL)
                if not os.path.exists(_mks):
                    logging.info('Error creating mks file for "%s".' % n)
                else:
                    i += 1
                    logging.info("Done (%s/%s)." % (i, _l))
            else:
                logging.info("No ass file found in " + _p)
            shutil.rmtree(p)
        else:
            logging.info("No subsetted directory in %s" % p)


if __name__ == "__main__":
    main(sys.argv[1:])
